@@ -1071,3 +1071,131 @@ class TestExpandVars:
             "foo": "bar",
             "wget_params": '--header="Referer: https://radiocut.fm/"',
         }
+
+
+class TestFileAwareEnv:
+    @pytest.fixture
+    def fa_env(self):
+        return environs.FileAwareEnv()
+
+    @pytest.fixture
+    def set_env_file(self, tmp_path, set_env):
+        def _set_env_file(env_key, value, suffix="_FILE"):
+            # create file with contents
+            file = tmp_path / f"{env_key}_secret"
+            file.write_text(value)
+            # set env var with path to file
+            file_env_key = f"{env_key}{suffix}"
+            set_env(
+                {
+                    file_env_key: str(file),
+                }
+            )
+
+        return _set_env_file
+
+    def test_read_from_file(self, fa_env, set_env_file):
+        set_env_file("KEY", "value from file")
+        assert fa_env.str("KEY") == "value from file"
+
+    def test_read_from_file_overrides_key(self, fa_env, set_env_file, set_env):
+        set_env_file("KEY", "value from file")
+        set_env(
+            {
+                "KEY": "value from env",
+            }
+        )
+        assert fa_env.str("KEY") == "value from file"
+
+    def test_read_from_non_existent_file(self, fa_env, set_env):
+        """When an env var exists that points to a file, that file should exist."""
+        set_env(
+            {
+                "KEY_FILE": "/path/to/non_existent/file",
+            }
+        )
+
+        with pytest.raises(
+            ValueError, match="The value of KEY_FILE must be a readable file path."
+        ):
+            fa_env.str("KEY")
+
+    def test_read_from_file_path_is_not_a_file(self, fa_env, tmp_path, set_env):
+        dir_path = tmp_path / "directory"
+        dir_path.mkdir()
+        set_env(
+            {
+                "KEY_FILE": str(dir_path),
+            }
+        )
+
+        with pytest.raises(
+            ValueError, match="The value of KEY_FILE must be a readable file path."
+        ):
+            fa_env.str("KEY")
+
+    def test_read_from_file_path_is_unreadable(self, fa_env, tmp_path, set_env):
+        file_path = tmp_path / "secret_file"
+        file_path.write_text("value from file")
+        file_path.chmod(0o000)
+        set_env(
+            {
+                "KEY_FILE": str(file_path),
+            }
+        )
+
+        with pytest.raises(
+            ValueError, match="The value of KEY_FILE must be a readable file path."
+        ):
+            assert fa_env.str("KEY")
+
+    def test_read_from_file_types(self, fa_env, set_env_file):
+        set_env_file("KEY_BOOL", "true")
+        set_env_file("KEY_INT", "123")
+        set_env_file("KEY_FLOAT", "12.34")
+        set_env_file("KEY_DECIMAL", "12.345")
+        set_env_file("KEY_LIST", "foo,bar")
+        set_env_file("KEY_TIMEDELTA", "123")
+        # etc
+
+        assert fa_env.bool("KEY_BOOL") is True
+        assert fa_env.int("KEY_INT") == 123
+        assert fa_env.float("KEY_FLOAT") == 12.34
+        assert fa_env.decimal("KEY_DECIMAL") == Decimal("12.345")
+        assert fa_env.list("KEY_LIST") == ["foo", "bar"]
+        assert fa_env.timedelta("KEY_TIMEDELTA") == dt.timedelta(seconds=123)
+
+    def test_request_suffixed_env_var(self, fa_env, set_env_file):
+        """The env var with the _FILE suffix should be usable as a normal var."""
+        set_env_file("KEY", "value from file")
+
+        # use it as a path
+        file_as_path = fa_env.path("KEY_FILE")
+        assert file_as_path.exists()
+
+        # use it as a string
+        file_as_str = fa_env.str("KEY_FILE")
+        assert str(file_as_path) == file_as_str
+
+    def test_expand_value_from_file(self, set_env, set_env_file):
+        fa_env = environs.FileAwareEnv(expand_vars=True)
+
+        set_env({"DATABASE_URL": "postgres://user:${PASSWORD}@host:5432/dbname"})
+        set_env_file("PASSWORD", "secret")
+
+        assert fa_env.str("DATABASE_URL") == "postgres://user:secret@host:5432/dbname"
+
+    def test_read_from_file_fall_back_to_default(self, fa_env, set_env_file):
+        assert fa_env.str("KEY", default="default value") == "default value"
+
+    def test_different_env_var_suffix(self, set_env_file):
+        # create env var KEY_SECRETFILE pointing to a file
+        set_env_file("KEY", "value from file", suffix="_SECRETFILE")
+
+        default_fa_env = environs.FileAwareEnv()
+        with pytest.raises(environs.EnvError):
+            # using the default suffix will not work
+            default_fa_env.str("KEY")
+
+        custom_fa_env = environs.FileAwareEnv(file_suffix="_SECRETFILE")
+        assert custom_fa_env.str("KEY") == "value from file"
